
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include "defs.h"

char memBase[32768];

void help(void);
void panic(const char *ctl, ...);
void show(const void *base, int bytes);

int
main(int ac, char **av)
{
    help();

    for (;;) {
	char buf[256];
	char c = 0;
	int v = 0;
	int a = 0;

	printf("mem> ");
	fflush(stdout);

	if (fgets(buf, sizeof(buf), stdin) == NULL)
	    break;

	sscanf(buf, "%c", &c);

	switch(c) {
	case 'h':
	case '?':
	    help();
	    break;
	case 'q':
	    puts("ok");
	    exit(0);
	case 'a':
	    sscanf(buf, "%c %d", &c, &v);
	    if (v) {
		void *ptr = malloc(v);
		printf("Allocate %d\t%lx\n", v, (long)ptr);
		if (ptr)
		    memset(ptr, -1, v);
	    }
	    break;
	case 'f':
	    sscanf(buf, "%c %x", &c, &a);
	    if (a) {
		free((void *)a);
	    }
	    break;
	case 'r':
	    sscanf(buf, "%c %x %d", &c, &a, &v);
	    if (v) {
		void *ptr = realloc((void *)a, v);
		printf("Reallocate addr=%lx bytes=%d\n", (long)ptr, v);
		show(ptr, v);
	    }
	    break;
	case 'w':
	    sscanf(buf, "%c %x %d", &c, &a, &v);
	    *(char *)a = v;
	    break;
	case 's':
	    sscanf(buf, "%c %x %d", &c, &a, &v);
	    if (v) {
		show((void *)a, v);
	    }
	    break;
	}
    }
}

void 
help(void)
{
    printf("%s",
	"h/?          - help\n"
	"q            - quit\n"
	"a bytes      - allocate\n"
	"f addr       - free\n"
	"r addr bytes - realloc\n"
	"w addr char  - write\n"
	"s addr bytes - show contents\n"
    );
}

void 
show(const void *base, int bytes)
{
    int count = 0;

    while (bytes) {
	if ((count & 15) == 0)
	    printf("\n%08lx ", (long)base);
	printf(" %3d", *(unsigned char *)base);
	++count;
	--bytes;
	base = (char *)base + 1;
    }
    printf("\n");
}

void
panic(const char *ctl, ...)
{
    va_list va;

    va_start(va, ctl);
    va_end(va);
    vfprintf(stderr, ctl, va);
    fflush(stderr);
    abort();
}

